c Version 2
c July 29, 2011
c B-mode spectrum due to Faraday rotation of E-mode.
c By Levon Pogosian(levon@sfu.ca) and Amit Yadav (ayadav@ias.edu)
c Uses file window_functions.dat as input. 
c Creates file clbb.d with two columns: 
c the first column is the multipole l: 2<l<10000
c the second column is l(l+1)C_l^{BB}/2\pi in units of (\mu K)^2
c Conventions follow those in arXiv:1106.1438

      program driver
      implicit none                
      INTEGER  i,il,nk0,nl0,nk,nl,ik,i_change,lout
      parameter(nl0=420,nk0=500)

      double precision d0hi,d0lo,pi,hubble

      parameter (d0hi=1.0d40,d0lo=1.0d40)
      parameter (pi=3.14159265359d0,hubble=0.72d0)

      double precision win(nk0,nl0),ak(nk0),wink(nk0) 
      integer l(nl0)

      double precision bnorm,n1,n2,omegabg,fcmb,omega_max,beff
      double precision akmin,akmax,dcl,cl,apowerb,xk
	double precision kappa,k_diss,k_I,lpeak
c	double precision gammln,gma

      double precision clb(nl0),clbpr(nl0),xl(nl0),ol,ocl
      double precision olm,oclm,slope

      open(unit=11,file='clbb.d',
     &     status='unknown',form='formatted')

      open(unit=9,file='window_functions.dat',
     &     status='old',form='formatted')

      read(9,*)nl,nk
      write(*,*) 'nl=',nl,' nk =',nk

      do ik=1,nk
      read(9,*)i,ak(ik)
      enddo      

      do il=1,nl
      read(9,*)l(il)
      enddo   

      do ik=1,nk
      do il=1,nl
      read(9,*)win(ik,il)
      enddo    
      enddo
      close(9)

      write(*,*)'Enter the CMB frequency in GHz'
      read(*,*)fcmb
      write(*,*)'Enter Omega_Bg'
      read(*,*)omegabg

	beff=3.25d-6*sqrt(omegabg)
	write(*,*)'This Omega_Bg corresponds to B_eff=',beff

	k_diss=0.043d0*sqrt(hubble/omegabg)

      write(*,*)'k_diss for this Omega_Bg is',k_diss,' 1/Mpc'
	
c      write(*,*)'Enter the magnetic spectral index (2n) at k<k_I'
c      read(*,*)n1
	
c	gma=dexp(gammln(n1/2.0d0+1.0d0))
	
c	k_diss=0.043d0*sqrt((2.0d0*pi)**n1*hubble/
c     &(omegabg*gma))

c      write(*,*)'Dissipation scale k_diss=',k_diss,' 1/Mpc'

      write(*,*)'Enter the inertial scale k_I (< k_diss) in 1/Mpc:'
      read(*,*)k_I

      write(*,*)'Enter the magnetic spectral index (2n) at k<k_I'
      read(*,*)n1

	if(k_diss.le.k_I) then
       write(*,*)'Your k_I is > k_diss.'
	 omega_max=hubble*0.043d0**2/k_I**2
	 write(*,*)'Maximum Omega_Bg for your k_I is',omega_max
10	 continue
	 write(*,*)'Keep your k_I and set Omega_Bg=Omega_max? (0)'
	 write(*,*)'Or keep your Omega_Bg and set k_I=k_diss? (1)'
	 read(*,*)i_change
	 if(i_change.eq.0) then
	  omegabg=omega_max
	  k_diss=k_I
	 endif
	 if(i_change.eq.1) then
	  k_I=k_diss
	 endif	 
	 if(i_change.ne.1.and.i_change.ne.0) then
	  write(*,*)'Bad entry. Try again:'
	  goto 10
	 endif 
	 n2=n1
      else
	 write(*,*)'Enter the magnetic spectral index (2n`)at k>k_I'
       read(*,*)n2
	endif
	
	kappa=1.0d0+(n1/n2)*((k_diss/k_I)**n2-1.0d0)

      write(*,*)'kappa=',kappa	

      bnorm=1.126d4*(omegabg/kappa)
     &*(n1/5.0d0)*(90.0d0/fcmb)**4.0d0

c set the range of integral over k
      akmin=ak(1)
      akmax=ak(nk)
	      
	lpeak=k_diss*1.5d4
	
	write(*,*)'Parameters of the magnetic spectrum:'
	write(*,*)omegabg,n1,n2,k_I,k_diss
	write(*,*)'------------------------------------'
	write(*,*)	
	write(*,*)'Evaluating C_l^BB'
	write(*,*)'Your k_diss implies approx lpeak=',lpeak
2	continue
	write(*,*)'Enter the maximum l (<10000) to output C_l^BB'
	read(*,*)lout
	if(lout.gt.10000) goto 2

c start the loop over il
      do il=1,nl

      cl=0.0d0
      dcl=0.0d0

      do ik=2,nk

      wink(ik)=win(ik,il)       
      xk=ak(ik)

	if(ak(ik).gt.k_diss) then
	 apowerb=0.0d0
      else
	 if(ak(ik).le.k_I) then
        apowerb=bnorm*(xk/k_I)**n1
	 else
	  apowerb=bnorm*(xk/k_I)**n2
	 endif
	endif
	
      dcl=wink(ik)*apowerb*Abs(ak(ik-1)-ak(ik))/xk
      
      cl=cl+dcl

      enddo

      cl=(2.0d0/pi)*cl
     & *dble(l(il)*(l(il)+1))/(2.d0*pi)
      
      clb(il)=cl
      xl(il)=dble(l(il))

      enddo    

      call spline(xl,clb,nl,d0hi,d0lo,clbpr)

      do il=2,lout

      ol=dble(il)

      call splint(xl,clb,clbpr,nl,ol,ocl)

      write(11,*)ol,abs(ocl)
      enddo

      close(11)

      stop
      end

      FUNCTION gammln(xx)
      DOUBLE PRECISION gammln,xx
      INTEGER j
      DOUBLE PRECISION ser,stp,tmp,x,y,cof(6)
      SAVE cof,stp
      DATA cof,stp/76.18009172947146d0,-86.50532032941677d0,
     *24.01409824083091d0,-1.231739572450155d0,.1208650973866179d-2,
     *-.5395239384953d-5,2.5066282746310005d0/
      x=xx
      y=x
      tmp=x+5.5d0
      tmp=(x+0.5d0)*log(tmp)-tmp
      ser=1.000000000190015d0
      do 11 j=1,6
        y=y+1.d0
        ser=ser+cof(j)/y
11    continue
      gammln=tmp+log(stp*ser/x)
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software 0@.1Y..

      SUBROUTINE splint(xa,ya,y2a,n,x,y)
      INTEGER n
      double precision x,y,xa(n),y2a(n),ya(n)
      INTEGER k,khi,klo
      double precision a,b,h
      klo=1
      khi=n
1     if (khi-klo.gt.1) then
        k=(khi+klo)/2
        if(xa(k).gt.x)then
          khi=k
        else
          klo=k
        endif
      goto 1
      endif
      h=xa(khi)-xa(klo)
      if (h.eq.0.d0) pause 'bad xa input in splint'
      a=(xa(khi)-x)/h
      b=(x-xa(klo))/h
      y=a*ya(klo)+b*ya(khi)+((a**3-a)*y2a(klo)+(b**3-b)*y2a(khi))*(h**
     *2)/6.d0
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software 0@.1Y..

      SUBROUTINE spline(x,y,n,yp1,ypn,y2)
      INTEGER n,NMAX
      real*8 yp1,ypn,x(n),y(n),y2(n)
      PARAMETER (NMAX=10010)
      INTEGER i,k
      real*8 p,qn,sig,un,u(NMAX)
      if (yp1.gt..99d30) then
        y2(1)=0.d0
        u(1)=0.d0
      else
        y2(1)=-0.5d0
        u(1)=(3.d0/(x(2)-x(1)))*((y(2)-y(1))/(x(2)-x(1))-yp1)
      endif
      do 11 i=2,n-1
        sig=(x(i)-x(i-1))/(x(i+1)-x(i-1))
        p=sig*y2(i-1)+2.d0
        y2(i)=(sig-1.d0)/p
        u(i)=(6.d0*((y(i+1)-y(i))/(x(i+
     *1)-x(i))-(y(i)-y(i-1))/(x(i)-x(i-1)))/(x(i+1)-x(i-1))-sig*
     *u(i-1))/p
11    continue
      if (ypn.gt..99d30) then
        qn=0.d0
        un=0.d0
      else
        qn=0.5d0
        un=(3.d0/(x(n)-x(n-1)))*(ypn-(y(n)-y(n-1))/(x(n)-x(n-1)))
      endif
      y2(n)=(un-qn*u(n-1))/(qn*y2(n-1)+1.d0)
      do 12 k=n-1,1,-1
        y2(k)=y2(k)*y2(k+1)+u(k)
12    continue
      return
C  (C) Copr. 1986-92 Numerical Recipes Software =$j*m,).
      END

